# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"__all__": "01_io.ipynb",
         "bgr2rgb": "00_utils.ipynb",
         "bgr2hsv": "00_utils.ipynb",
         "lapply": "00_utils.ipynb",
         "resize": "00_utils.ipynb",
         "read_video": "01_io.ipynb",
         "read_frames": "01_io.ipynb",
         "capture": "01_io.ipynb",
         "get_target_frames": "01_io.ipynb"}

modules = ["utils.py",
           "io.py"]

doc_url = "https://rsomani95.github.io/videoutils/"

git_url = "https://github.com/rsomani95/videoutils/tree/master/"

def custom_doc_links(name): return None